﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 自キャラクタのショット群を扱うクラスです。
    /// </summary>
    class CShotGroup
    {

        /// <summary>
        /// ショット 0の弾数。
        /// </summary>
        private int[] cshot0num = new int[] { 2, 4 };

        /// <summary>
        /// ショット 0の初期 X座標。
        /// </summary>
        private int[] cshot0pos_x = new int[] { -10, 10, -30, 30 };

        /// <summary>
        /// ショット 1の初期 Y座標。
        /// </summary>
        private int[] cshot0pos_y = new int[] { -30, -30, -10, -10 };

        /// <summary>
        /// 自キャラクタのショット群。
        /// </summary>
        private CShot[] cshots = new CShot[Setting.CSHOT_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public CShotGroup()
        {
            for (int i = 0; i < this.cshots.Length; i++)
            {
                this.cshots[i] = new CShot();
            }
        }

        /// <summary>
        /// ショットにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>ショット</returns>
        public CShot this[int index]
        {
            get
            {
                return this.cshots[index];
            }
        }

        /// <summary>
        /// 自キャラクタのショットの数を返します。
        /// </summary>
        /// <returns>ショットの数</returns>
        public int length()
        {
            return this.cshots.Length;
        }
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.cshots.Length; i++)
                this.cshots[i].initialize();
        }
        
        /// <summary>
        /// 自キャラクタの空いているショットを探します。
        /// </summary>
        /// <returns>空いているショット。無い場合は null</returns>
        private CShot search_cshot()
        {
            for (int i = 0; i < this.cshots.Length; i++)
            {
                CShot cshot = this.cshots[i];
                if (cshot.flag == 0)
                    return cshot;
            }

            return null;
        }

        /// <summary>
        /// 通常ショットを登録します。
        /// </summary>
        private void ch0_shot_pattern()
        {
            int shotNum = this.cshot0num[(GV.ch.power < 200) ? 0 : 1];
            for(int i = 0; i < shotNum; i ++)
            {
                CShot cshot = this.search_cshot();
                if(cshot != null)
                {
                    cshot.flag = 1;
                    cshot.cnt = 0;
                    cshot.angle = -Math.PI / 2;
                    cshot.spd = 20.0;
                    cshot.x = GV.ch.x + this.cshot0pos_x[i];
                    cshot.y = GV.ch.y + this.cshot0pos_y[i];
                    cshot.power = 23;
                    cshot.knd = 0;
                }
            }

            MusicManager musicManager = MusicManager.GetInstance();
            musicManager.se_flag[2] = 1; // 発射音をオン
        }

        /// <summary>
        /// 低速通常ショットを登録します。
        /// </summary>
        private void ch1_shot_pattern()
        {
            int shotNum = this.cshot0num[(GV.ch.power < 200) ? 0 : 1];
            for(int i = 0; i < shotNum; i ++)
            {
                CShot cshot = this.search_cshot();
                if(cshot != null)
                {
                    cshot.flag = 1;
                    cshot.cnt = 0;
                    cshot.angle = -Math.PI / 2;
                    cshot.spd = 20.0;
                    cshot.x = GV.ch.x + this.cshot0pos_x[i] / 3; // 低速中なら位置を中心側へ
                    cshot.y = GV.ch.y + this.cshot0pos_y[i] / 2;
                    cshot.power = 23;
                    cshot.knd = 0;
                }
            }

            MusicManager musicManager = MusicManager.GetInstance();
            musicManager.se_flag[2] = 1; // 発射音をオン
        }

        /// <summary>
        /// ショットを登録します。
        /// </summary>
        private void enter_shot()
        {
            KeyManager keyManager = KeyManager.GetInstance();
            if (keyManager.ChckStatePad(GV.configpad.shot) > 0) // ショットボタンが押されていたら
            {
                GV.ch.increment_shot_cnt();
                if (GV.ch.shot_cnt % 3 == 0) // 3カウントに 1回
                {
                    if (keyManager.ChckStatePad(GV.configpad.slow) > 0) // 低速移動中なら
                        this.ch1_shot_pattern();
                    else
                        this.ch0_shot_pattern();
                }
            }
            else
                GV.ch.reset_shot_cnt();
        }

        /// <summary>
        /// ショットの移動計算をします。
        /// </summary>
        private void calc_shot()
        {
            for (int i = 0; i < this.cshots.Length; i++)
            {
                CShot cshot = this.cshots[i];
                if (cshot.flag == 1)
                {
                    // 画面外判定用の変数(11, 55はショットの画像サイズ)
                    int dranx = (int)(cshot.spd + 11 / 2);
                    int drany = (int)(cshot.spd + 55 / 2);

                    cshot.x += Math.Cos(cshot.angle) * cshot.spd;
                    cshot.y += Math.Sin(cshot.angle) * cshot.spd;
                    cshot.cnt++;

                    if (cshot.x < -dranx || Setting.FIELD_MAX_X + dranx < cshot.x || cshot.y < -drany || Setting.FIELD_MAX_Y + drany < cshot.y) // 画面から外れたら
                        cshot.flag = 0;
                }
            }
        }

        /// <summary>
        /// 自キャラクタのショットのメイン処理。
        /// </summary>
        public void cshot_main()
        {
            this.calc_shot(); // ショットの軌道計算
            this.enter_shot(); // ショットの登録
        }
    }
}
