﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 点を表すクラスです。
    /// </summary>
    class Pt
    {
        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public Pt()
        {
            ; // 何もしない
        }

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="x">X座標</param>
        /// <param name="y">Y座標</param>
        public Pt(double x, double y)
        {
            this.x = x;
            this.y = y;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.x = 0.0;
            this.y = 0.0;
        }

        /// <summary>
        /// 座標回転します。
        /// (x0, y0)から (mx, my)を基準に ang角回転した角度に設定します。
        /// </summary>
        /// <param name="x0">回転元 X座標</param>
        /// <param name="y0">回転元 Y座標</param>
        /// <param name="mx">回転基準 X座標</param>
        /// <param name="my">回転基準 Y座標</param>
        /// <param name="ang">角度</param>
        public void conv_pos0(double x0, double y0, double mx, double my, double ang)
        {
            double ox = x0 - mx;
            double oy = y0 - my;
            this.x = ox * Math.Cos(ang) + oy * Math.Sin(ang);
            this.y = -ox * Math.Sin(ang) + oy * Math.Cos(ang);
            this.x += mx;
            this.y += my;
        }

        /// <summary>
        /// 点が円の中にあるかどうかを返します。
        /// </summary>
        /// <param name="p1">調べる点</param>
        /// <param name="r">点の半径</param>
        /// <returns>円の中にある場合は true</returns>
        private bool question_point_and_circle(Pt p, double r)
        {
            double dx = p.x - this.x;
            double dy = p.y - this.y;

            if (dx * dx + dy * dy < r * r)
                return true;
            else
                return false;
        }

        /// <summary>
        /// 3点の為す角度を返します。
        /// </summary>
        /// <param name="pt0">1点目</param>
        /// <param name="pt1">2点目</param>
        /// <param name="rpt">3点目</param>
        private static double get_theta(Pt pt0, Pt pt1, Pt rpt)
        {
            /* ベクトル C→P と C→Q のなす角θおよび回転方向を求める．*/
            Vector2 c = new Vector2(); /* 入力データ */
            Vector2 p = new Vector2();
            Vector2 q = new Vector2();
            Vector2 cp = new Vector2(); /* ベクトル C→P */
            Vector2 cq = new Vector2(); /* ベクトル C→Q */
            double s; /* 外積：(C→P) × (C→Q) */
            double t; /* 内積：(C→P) ・ (C→Q) */
            double theta; /* θ (ラジアン) */

            /* c，p，q を所望の値に設定する．*/
            c.x = pt0.x;
            c.y = pt0.y;
            p.x = pt1.x;
            p.y = pt1.y;
            q.x = rpt.x;
            q.y = rpt.y;

            /* 回転方向および角度θを計算する．*/
            cp.Vector2Diff(p, c); /* cp ← p - c   */
            cq.Vector2Diff(q, c); /* cq ← q - c   */
            s = Vector2.Vector2OuterProduct(cp, cq); /* s ← cp × cq */
            t = Vector2.Vector2InnerProduct(cp, cq); /* t ← cp ・ cq */
            theta = Math.Atan2(s, t);
            return theta;
        }

        /// <summary>
        /// この点と線分との距離を返します。
        /// </summary>
        /// <param name="x1">線の始点 X座標</param>
        /// <param name="y1">線の始点 Y座標</param>
        /// <param name="x2">線の終点 X座標</param>
        /// <param name="y2">線の終点 Y座標</param>
        /// <returns>距離</returns>
        double get_distance(double x1, double y1, double x2, double y2)
        {
            double dx = (x2 - x1);
            double dy = (y2 - y1);
            double a = dx * dx + dy * dy;
            double b = dx * (x1 - this.x) + dy * (y1 - this.y);
            double t = -b / a;
            if (t < 0)
                t = 0;
            if (t > 1)
                t = 1;
            double tx = x1 + dx * t;
            double ty = y1 + dy * t;
            double distance = Math.Sqrt((x - tx) * (x - tx) + (y - ty) * (y - ty));
            return distance;
        }
        /// <summary>
        /// 長方形と円との当たり判定をします。
        /// </summary>
        /// <param name="pt">長方形</param>
        /// <param name="r">この円のサイズ</param>
        public bool hitjudge_square_and_circle(Pt[] pt, double r)
        {
            // 円の中に長方形の 4点のうちどれかがあるという判定
            for (int i = 0; i < 4; i++)
            {
                if (this.question_point_and_circle(pt[i], r))
                    return true;
            }
            
            // 長方形の中に物体が入り込んでいるかどうかを判定
            double theta = get_theta(pt[0], pt[1], this); // 3点の為す角 1
            double theta2 = get_theta(pt[2], pt[3], this); // 3点の為す角 2
            if (0 <= theta && theta <= Math.PI / 2 && 0 <= theta2 && theta2 <= Math.PI / 2)
                return true;

            // 線分の上に物体が乗っているかどうかを判定
            for (int i = 0; i < 4; i++)
            {
                double distance = this.get_distance(pt[i].x, pt[i].y, pt[(i + 1) % 4].x, pt[(i + 1) % 4].y); // 線分と点との距離を求める
                if (distance < r)
                    return true;
            }

            return false;
        }
    }
}
