﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// グラフィックスを管理するクラスです。
    /// </summary>
    class GraphManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static GraphManager manager = new GraphManager();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GraphManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static GraphManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// 画像を回転描画します。
        /// </summary>
        /// <param name="xd">中心 X座標</param>
        /// <param name="yd">中心 Y座標</param>
        /// <param name="ExRate">拡大率</param>
        /// <param name="Angle">描画角度</param>
        /// <param name="GrHandle">グラフィックの識別番号</param>
        /// <param name="TransFlag">1のとき透明色を有効にする</param>
        /// <param name="TurnFlag">1のとき画像を反転する</param>
        /// <returns></returns>
        private int DrawRotaGraphF(double xd, double yd, double ExRate, double Angle, int GrHandle, int TransFlag)
        {
            return DX.DrawRotaGraphF((float)xd, (float)yd, ExRate, Angle, GrHandle, TransFlag);
        }
        
        /// <summary>
        /// 画像を変形描画します。
        /// </summary>
        /// <param name="x1">左上の頂点座標</param>
        /// <param name="y1">左上の頂点座標</param>
        /// <param name="x2">右上の頂点座標</param>
        /// <param name="y2">右上の頂点座標</param>
        /// <param name="x3">右下の頂点座標</param>
        /// <param name="y3">右下の頂点座標</param>
        /// <param name="x4">左下の頂点座標</param>
        /// <param name="y4">左下の頂点座標</param>
        /// <param name="GrHandle">描画するグラフィックのハンドル</param>
        /// <param name="TransFlag">1のとき透明色を有効にする</param>
        /// <returns></returns>
        private int DrawModiGraphF(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4, int GrHandle, int TransFlag)
        {
            return DX.DrawModiGraphF((float)x1, (float)y1, (float)x2, (float)y2, (float)x3, (float)y3, (float)x4, (float)y4, GrHandle, TransFlag);
        }

        /// <summary>
        /// エフェクト群を描画します。
        /// </summary>
        /// <param name="knd">エフェクトの種類</param>
        private void graph_effect(int knd)
        {
            for (int i = 0; i < GV.effects.length(); i++)
            {
                Effect effect = GV.effects[i];
                if (effect.flag > 0 && effect.knd == knd)
                {
                    if (effect.eff == 1) // エフェクトが光エフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, (int)effect.brt);
                    if (effect.eff == 2) // エフェクトがαエフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ALPHA, (int)effect.brt);

                    this.DrawRotaGraphF(effect.x + Setting.FX + GV.dn.x, effect.y + Setting.FY + GV.dn.y,
                        effect.r, effect.ang, effect.img, 1);

                    if (effect.eff == 1 || effect.eff == 2)
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                }
            }
        }

        /// <summary>
        /// 子どもを描画します。
        /// </summary>
        private void graph_child()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.children.length(); i++)
            {
                Child child = GV.children[i];
                if(child.flag == 1)
                {
                    this.DrawRotaGraphF(child.x + Setting.FX + GV.dn.x, child.y + Setting.FY + GV.dn.y,
                        child.range, MyMath.PI2 * (GV.count % 60) / 60,
                        imageManager.img_etc[6], 1);
                }
            }
        }

        /// <summary>
        /// アイテムを描画します。
        /// </summary>
        private void graph_item()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.items.length(); i++)
            {
                Item item = GV.items[i];
                if (item.flag == 1)
                {
                    this.DrawRotaGraphF(item.x + Setting.FX + GV.dn.x, item.y + Setting.FY + GV.dn.y,
                        item.r, MyMath.PI2 * (GV.count % 120) / 120,
                        imageManager.img_item[item.knd, 1], 1);
                    this.DrawRotaGraphF(item.x + Setting.FX + GV.dn.x, item.y + Setting.FY + GV.dn.y,
                        item.r * 0.8, - MyMath.PI2 * (GV.count % 120) / 120,
                        imageManager.img_item[item.knd, 1], 1);
                    this.DrawRotaGraphF(item.x + Setting.FX + GV.dn.x, item.y + Setting.FY + GV.dn.y,
                        item.r, 0,
                        imageManager.img_item[item.knd, 0], 1);
                }
            }
        }

        /// <summary>
        /// ボスのエフェクトを描画します。
        /// </summary>
        private void graph_boss_effect()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ALPHA, 150);

            // ボス背景の円形の内側
            this.DrawRotaGraphF(
                GV.boss.dx + Setting.FX,
                GV.boss.dy + Setting.FY,
                (0.4 + 0.05 * Math.Sin(2 * Math.PI / 360 * (GV.count % 360))) * 3,
                MyMath.PI2 * (GV.count % 580) / 580,
                imageManager.img_etc[5], 1);
            // ボス背景の円形の外側
            this.DrawRotaGraphF(
                GV.boss.dx + Setting.FX,
                GV.boss.dy + Setting.FY,
                (0.5 + 0.1 * Math.Sin(MyMath.PI2 / 360 * (GV.count % 360))) * 2,
                2 * Math.PI * (GV.count % 340) / 340,
                imageManager.img_etc[2], 1);

            // 奥にうっすら表示される回転体
            this.DrawRotaGraphF(
                GV.boss.dx + 60 * Math.Sin(MyMath.PI2 / 153 * (GV.count % 153)) + Setting.FX,
                GV.boss.dy + 80 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)) + Setting.FY,
                0.4 + 0.05 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)),
                2 * Math.PI * (GV.count % 30) / 30,
                imageManager.img_etc[6], 1);
            this.DrawRotaGraphF(
                GV.boss.dx + 60 * Math.Sin(MyMath.PI2 / 200 * ((GV.count + 20) % 200)) + Setting.FX,
                GV.boss.dy + 80 * Math.Sin(MyMath.PI2 / 177 * ((GV.count + 20) % 177)) + Setting.FY,
                0.3 + 0.05 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)),
                2 * Math.PI * (GV.count % 35) / 35,
                imageManager.img_etc[6], 1);

            DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);

            // 手前に表示される回転体
            this.DrawRotaGraphF(
                GV.boss.dx + 60 * Math.Sin(MyMath.PI2 / 230 * ((GV.count + 40) % 230)) + Setting.FX,
                GV.boss.dy + 80 * Math.Sin(MyMath.PI2 / 189 * ((GV.count + 40) % 189)) + Setting.FY,
                0.6 + 0.05 * Math.Sin(MyMath.PI2 / 120 * (GV.count % 120)),
                2 * Math.PI * (GV.count % 40) / 40,
                imageManager.img_etc[6], 1);
        }

        /// <summary>
        /// ボスを描画します。
        /// </summary>
        private void graph_boss()
        {
            if (GV.boss.flag == 0)
                return;

            this.graph_boss_effect(); // ボスのエフェクト描画する

            ImageManager imageManager = ImageManager.GetInstance();
            this.DrawRotaGraphF(GV.boss.dx + Setting.FX + GV.dn.x, GV.boss.dy + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_dot_riria[0], 1);
            if (GV.boss.hp_max == 0)
            {
                Debug.WriteLine("graph_boss内 0割り");
                return;
            }
            for (int i = 0; i < Setting.FMX * 0.98 * GV.boss.hp / GV.boss.hp_max; i++) // HPを表示する
            {
                if (GV.boss.back_knd[GV.boss.knd] == 1)
                    DX.DrawGraph(3 + Setting.FX + i + GV.dn.x, 2 + Setting.FY + GV.dn.y, imageManager.img_etc[7], 0);
                else
                    DX.DrawGraph(3 + Setting.FX + i + GV.dn.x, 2 + Setting.FY + GV.dn.y, imageManager.img_etc[1], 0);
            }
        }

        /// <summary>
        /// 敵キャラクタ群を描画します。
        /// </summary>
        private void graph_enemy()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.enemys.length(); i++)
            {
                Enemy enemy = GV.enemys[i];
                if (enemy.flag == 1)
                    this.DrawRotaGraphF(enemy.x + Setting.FX + GV.dn.x, enemy.y + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_enemy[0, enemy.img], 1);
            }
        }

        /// <summary>
        /// 自キャラクタのショットを描画します。
        /// </summary>
        private void graph_cshot()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.cshots.length(); i++)
            {
                CShot cshot = GV.cshots[i];
                if (cshot.flag > 0)
                    this.DrawRotaGraphF(cshot.x + Setting.FX + GV.dn.x, cshot.y + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_cshot[cshot.knd], 1);
            }
        }

        /// <summary>
        /// 自キャラクタを描画します。
        /// </summary>
        private void graph_ch()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            KeyManager keyManager = KeyManager.GetInstance();
            double sx;
            double sy;
            double ny = Math.Sin(2.0 * Math.PI * (GV.count % 50) / 50) * 3;
            double ang = 2.0 * Math.PI * (GV.count % 120) / 120;

            if (keyManager.CheckStatePad(GV.configpad.slow) > 0) // 低速移動中なら
            {
                sx = 15; // 引き寄せる
                sy = 15 + ny;
            }
            else
            {
                sx = 30; // 普通の位置に
                sy = 30 + ny;
            }

            // ボールの表示(dn.x, dn.yの加算はないけどOK?)
            this.DrawRotaGraphF(GV.ch.x - sx + Setting.FX, GV.ch.y + sy + Setting.FY + GV.dn.y, 1.0, ang, imageManager.img_chetc[2], 1);
            this.DrawRotaGraphF(GV.ch.x + sx + Setting.FX, GV.ch.y + sy + Setting.FY + GV.dn.y, 1.0, -ang, imageManager.img_chetc[2], 1);

            if (GV.ch.mutekicnt % 2 == 0) // 無敵中なら点滅
            {
                // 自キャラクタを表示
                this.DrawRotaGraphF(GV.ch.x + Setting.FX + GV.dn.x, GV.ch.y + Setting.FY + GV.dn.y, 1.0, 0.0, imageManager.img_ch[0, GV.ch.img], 1);
                if(keyManager.CheckStatePad(GV.configpad.slow) > 0) // 移動低速中なら当たり判定表示(dn.x, dn.yの加算はないけどOK?)
                    this.DrawRotaGraphF(GV.ch.x + Setting.FX, GV.ch.y + Setting.FY, 1.0, 2.0 * Math.PI * (GV.count % 120) / 120, imageManager.img_chetc[0], 1);
            }
        }

        /// <summary>
        /// 四角形を描画します。
        /// </summary>
        /// <param name="x0">点 0の X座標</param>
        /// <param name="y0">点 0の Y座標</param>
        /// <param name="x1">点 1の X座標</param>
        /// <param name="y1">点 1の Y座標</param>
        /// <param name="x2">点 2の X座標</param>
        /// <param name="y2">点 2の Y座標</param>
        /// <param name="x3">点 3の X座標</param>
        /// <param name="y3">点 3の Y座標</param>
        private void myDrawSquare(double x0, double y0, double x1, double y1, double x2, double y2, double x3, double y3)
        {
            DX.DrawTriangle((int)x0, (int)y0, (int)x1, (int)y1, (int)x2, (int)y2, DX.GetColor(255, 0, 0), 1);
            DX.DrawTriangle((int)x0, (int)y0, (int)x3, (int)y3, (int)x2, (int)y2, DX.GetColor(255, 0, 0), 1);
        }

        /// <summary>
        /// レーザーを描画します。
        /// </summary>
        private void graph_lazer()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawMode(DX.DX_DRAWMODE_BILINEAR); // 線形補間描画

            for (int i = 0; i < GV.lazers.length(); i++)
            {
                Lazer lazer = GV.lazers[i];
                if (lazer.flag > 0) // レーザーがオンなら
                {
                    DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                    // 発射位置のエフェクトを描画
                    this.DrawRotaGraphF(lazer.startpt.x + Setting.FX, lazer.startpt.y + Setting.FY,
                        1.0, 0, imageManager.img_lazer_moto[lazer.knd, lazer.col], 1);

                    // レーザーを描画
                    this.DrawModiGraphF(//レーザーを描画
                        lazer.disppts[0].x + Setting.FX, lazer.disppts[0].y + Setting.FY,
                        lazer.disppts[1].x + Setting.FX, lazer.disppts[1].y + Setting.FY,
                        lazer.disppts[2].x + Setting.FX, lazer.disppts[2].y + Setting.FY,
                        lazer.disppts[3].x + Setting.FX, lazer.disppts[3].y + Setting.FY,
                        imageManager.img_lazer[lazer.knd, lazer.col], 1);

                    DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);

                    //当たり判定範囲を表示
                    /*
                    this.myDrawSquare(
                        lazer.outpts[0].x + Setting.FX, lazer.outpts[0].y + Setting.FY,
                        lazer.outpts[1].x + Setting.FX, lazer.outpts[1].y + Setting.FY,
                        lazer.outpts[2].x + Setting.FX, lazer.outpts[2].y + Setting.FY,
                        lazer.outpts[3].x + Setting.FX, lazer.outpts[3].y + Setting.FY);
                     */
                }
            }

            DX.SetDrawMode(DX.DX_DRAWMODE_NEAREST); // 描画形式を戻す
        }

        /// <summary>
        /// ボスのアシスト画像を描画します。
        /// </summary>
        private void boss_assist_img()
        {
            ImageManager imageManager = ImageManager.GetInstance();

            if (GV.boss_shot.flag > 0)
            { // 弾幕データがオンなら
                for(int j = 0; j < GV.boss_shot.assists.length(); j++) { // その弾幕が持つ弾の最大数分ループ
                    Bullet assist = GV.boss_shot.assists[j];

                    if(assist.flag != 0){ // 弾データがオンなら
                        // 計算部
                        assist.x += Math.Cos(assist.angle) * assist.spd;
                        assist.y += Math.Sin(assist.angle) * assist.spd;
                        assist.cnt++;

                        int eff = 0;
                        double disp_angle;

                        if(assist.kaiten == 1)
                            disp_angle = MyMath.PI2 * (assist.cnt % 120) / 120;
                        else
                            disp_angle = assist.angle + Math.PI / 2;

                        if(assist.eff == 1)
                        {
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);
                            eff = 1;
                        }
                        if (assist.eff == 2)
                        {
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255 + MyMath.GetRand(100));
                            eff = 1;
                        }
                        if(assist.c_flag == 0)
                            DX.DrawRotaGraphF((float)assist.x, (float)assist.y,
                                1.0, disp_angle,
                                imageManager.img_bullet[assist.knd, assist.col], 1);
                        else
                            DX.DrawRotaGraph2F((float)assist.x, (float)assist.y,
                                (float)assist.cx, (float)assist.cy,
                                1.0, disp_angle,
                                imageManager.img_bullet[assist.knd, assist.col], 1);

                        if(eff == 1)
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕を描画します。
        /// </summary>
        private void graph_bullet()
        {
            double disp_angle; // 表示角度

            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawMode(DX.DX_DRAWMODE_BILINEAR); // 線形補間描画

            // 敵弾
            for (int i = 0; i < GV.shots.length(); i++) // 弾幕数分ループ
            {
                Shot shot = GV.shots[i];
                if (shot.flag > 0) // 弾幕データがオンなら
                {
                    BulletGroup bullets = shot.bullets;
                    for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                    {
                        Bullet bullet = bullets[j];
                        if (bullet.flag != 0) // 弾データがオンなら
                        {
                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                            if (GV.bullet_infos[bullet.knd].kaiten == 1) // 弾の種類が回転する弾の場合
                                disp_angle = MyMath.PI2 * (bullet.cnt % 120) / 120;
                            else
                                disp_angle = bullet.angle + Math.PI / 2;

                            this.DrawRotaGraphF(bullet.x + Setting.FX + GV.dn.x, bullet.y + Setting.FY + GV.dn.y,
                                1.0, disp_angle,
                                imageManager.img_bullet[bullet.knd, bullet.col], 1);

                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                        }
                    }
                }
            }

            this.boss_assist_img();

            // ボス弾
            if(GV.boss_shot.flag > 0) // 弾幕データがオンなら
            {
                BulletGroup bullets = GV.boss_shot.bullets;
                for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                {
                    Bullet bullet = bullets[j];
                    if (bullet.flag != 0) // 弾データがオンなら
                    {
                        int eff = 0;

                        if (bullet.kaiten == 1) // 弾が回転状態なら
                            disp_angle = MyMath.PI2 * (bullet.cnt % 120) / 120;
                        else
                            disp_angle = bullet.angle + Math.PI / 2;

                        if (bullet.eff_detail == 1) // 風雨にも負けず用弾エフェクト
                        {
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 100 + MyMath.GetRand(155));
                            eff = 1;
                            this.DrawRotaGraphF(bullet.x + Setting.FX + GV.dn.x, bullet.y + Setting.FY + GV.dn.y,
                                1.3, disp_angle,
                                imageManager.img_bullet[bullet.knd, bullet.col], 1);
                        }
                        if (bullet.eff == 1)
                        {
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);
                            eff=1;
                        }

                        if (bullet.eff == 2)
                        {
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255 + MyMath.GetRand(100));
                            eff=1 ;
                        }

                        this.DrawRotaGraphF(bullet.x + Setting.FX + GV.dn.x, bullet.y + Setting.FY + GV.dn.y,
                            1.0, disp_angle,
                            imageManager.img_bullet[bullet.knd, bullet.col], 1);

                        if (eff == 1)
                            DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                    }
                }
            }
            DX.SetDrawMode(DX.DX_DRAWMODE_NEAREST); // 描画形式を戻す
        }

        /// <summary>
        /// ステージタイトルを描画します。
        /// </summary>
        private void graph_stage_title()
        {
            StageTitle stage_title = GV.stage_title;
            if (stage_title.flag > 0)
            {
                DX.SetDrawBlendMode(DX.DX_BLENDMODE_ALPHA, stage_title.brt);
                DX.DrawGraph(120 + Setting.FX + GV.dn.x, 10 + Setting.FY + GV.dn.y, stage_title.img, 1);
                DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
            }
        }

        /// <summary>
        /// ボード上の情報を表示します。
        /// </summary>
        private void graph_board_states()
        {
            int score = GV.ch.score;
            int power = GV.ch.power;
            int graze = GV.ch.graze;
            int point = GV.ch.point;
            int money = GV.ch.money;

            ImageManager imageManager = ImageManager.GetInstance();

            for (int i = 0; i < 9; i++) // スコア表示
            {
                DX.DrawRotaGraph(625 - 15 * i, 30, 1.0, 0.0, imageManager.img_num[0, score % 10], 1);
                DX.DrawRotaGraph(625 - 15 * i, 50, 1.0, 0.0, imageManager.img_num[0, score % 10], 1);
                score /= 10;
            }

            for (int i = 0; i < GV.ch.num; i++) // 残機数表示
                DX.DrawGraph(499 + 12 * i, 63, imageManager.img_etc[8], 1);

            // パワー表示
            DX.DrawRotaGraph(547, 91, 0.9, 0.0, imageManager.img_num[0,power % 10], 1);
            power /= 10;
            DX.DrawRotaGraph(536, 91, 0.9, 0.0, imageManager.img_num[0,power % 10], 1);
            power /= 10;
            DX.DrawRotaGraph(513, 91, 1.0, 0.0, imageManager.img_num[0,power % 10], 1);
            DX.DrawString(522, 82, ".", GV.color[0]); // チョン

            for (int i = 0; i < 6; i++) // グレイズ表示
            {
                DX.DrawRotaGraph(578 - 14 * i, 111, 1.0, 0.0, imageManager.img_num[0, graze % 10], 1);
                graze /= 10;
            }

            for (int i = 0; i < 4; i++) // ポイント表示
            {
                DX.DrawRotaGraph(550 - 14 * i, 131, 1.0, 0.0, imageManager.img_num[0, point % 10], 1);
                point /= 10;
            }

            for (int i = 0; i < 6; i++) // マネー表示
            {
                DX.DrawRotaGraph(578 - 14 * i, 154, 1.0, 0.0, imageManager.img_num[0, money % 10], 1);
                money /= 10;
            }
        }

        /// <summary>
        /// ボードを描画します。
        /// </summary>
        private void graph_board()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawGraph(0, 0, imageManager.img_board[10], 0);
            DX.DrawGraph(0, 16, imageManager.img_board[11], 0);
            DX.DrawGraph(0, 464, imageManager.img_board[12], 0);
            DX.DrawGraph(416, 0, imageManager.img_board[20], 0);

            this.graph_board_states();
        }

        /// <summary>
        /// 開発用の文字を描画します。
        /// </summary>
        private void graph_develop()
        {
            DX.DrawString(0, 0, "" + GV.stage_count, DX.GetColor(255, 255, 255), DX.GetColor(255, 255, 255));
        }

        /// <summary>
        /// 描画します。
        /// </summary>
        public void graph_main()
        {
            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            GraphBack.graph_back_main(); // 背景描画メイン
            this.graph_effect(0); // 敵が死ぬエフェクト

            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_effect(4); // 喰らいボムのエフェクト

            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_child(); // 子どもの描画
            this.graph_item(); // アイテムの描画
            this.graph_boss(); // ボスの描画
            this.graph_enemy(); // 敵の描画
            this.graph_cshot(); // 自キャラクタのショットの描画

            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_ch(); // 自キャラクタの描画

            if (GV.bright_set.brt != 255) // 以降は暗くなる可能性がある
                DX.SetDrawBright(GV.bright_set.brt, GV.bright_set.brt, GV.bright_set.brt);

            this.graph_lazer(); // レーザーの描画
            this.graph_bullet(); // 弾の描画
            
            if (GV.bright_set.brt != 255) // 以降は明るさの影響を受けない
                DX.SetDrawBright(255, 255, 255);

            this.graph_effect(1); // ボムのエフェクト
            this.graph_effect(2); // ボム線のエフェクト
            this.graph_effect(3); // ボムキャラのエフェクト
            this.graph_stage_title(); // タイトルの描画
            this.graph_board(); // ボードの描画

            this.graph_develop(); // 開発用の描画
        }
    }
}
